# Copyright 1999-2009 University of Chicago
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Object definition for processing Java WS Core (version 2) usage packets.
"""

from javawscorev1packet import JavaWSCoreV1Packet

class JavaWSCoreV2Packet(JavaWSCoreV1Packet):
    """
    Java WS Core Packet (Version 2) packets are generated by GT 4.0.x (x >= 3)
    This packet adds an optional integer value and a list of services that
    were present (at start) and activated (at terminate) by the container.
    """
    def __init__(self, address, packet):
        JavaWSCoreV1Packet.__init__(self, address, packet)
        [self.optional_int, service_len] = self.unpack("ih")
        self.services = self.unpack_string(service_len)

    insert_statement = '''
            INSERT INTO java_ws_core_packets(
                component_code,
                version_code,
                send_time,
                ip_address,
                container_id,
                container_type,
                event_type,
                service_list,
                optional_val)
            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)'''

    def values(self, dbclass):
        """
        Return a values tuple which matches the parameters in the
        class's insert_statement.

        Arguments:
        self -- A JavaWSCoreV2Packet object

        Returns:
        Tuple containing
            (component_code, version_code, send_time, ip_address, container_id,
             container_type, event_type, service_list, optional_val)

        """
        return (
            self.component_code,
            self.packet_version,
            dbclass.Timestamp(*self.send_time),
            self.ip_address,
            self.container_id,
            self.container_type,
            self.event_type,
            self.services,
            self.optional_int)
